# Contributing to ubuntu-proxy-manager

A big welcome and thank you for considering contributing to ubuntu-proxy-manager and Ubuntu! It’s people like you that make it a reality for users in our community.

Reading and following these guidelines will help us make the contribution process easy and effective for everyone involved. It also communicates that you agree to respect the time of the developers managing and developing this project. In return, we will reciprocate that respect by addressing your issue, assessing changes, and helping you finalize your pull requests.

These are mostly guidelines, not rules. Use your best judgment, and feel free to propose changes to this document in a pull request.

## Quicklinks

- [Contributing to ubuntu-proxy-manager](#contributing-to-ubuntu-proxy-manager)
  - [Quicklinks](#quicklinks)
  - [Code of Conduct](#code-of-conduct)
  - [Getting Started](#getting-started)
    - [Issues](#issues)
    - [Pull Requests](#pull-requests)
  - [Contributing to the code](#contributing-to-the-code)
    - [Required dependencies and setup](#required-dependencies-and-setup)
    - [About the testsuite](#about-the-testsuite)
  - [Contributor Licence Agreement](#contributor-licence-agreement)
  - [Getting Help](#getting-help)

## Code of Conduct

We take our community seriously and hold ourselves and other contributors to high standards of communication. By participating and contributing to this project, you agree to uphold our [Code of Conduct](https://ubuntu.com/community/code-of-conduct).

## Getting Started

Contributions are made to this project via Issues and Pull Requests (PRs). A few general guidelines that cover both:

* To report security vulnerabilities, please use the advisories page of the repository and not a public bug report.
* Search for existing Issues and PRs before creating your own.
* We work hard to make sure issues are handled in a timely manner but, depending on the impact, it could take a while to investigate the root cause. A friendly ping in the comment thread to the submitter or a contributor can help draw attention if your issue is blocking.
* If you've never contributed before, see [this Ubuntu discourse post](https://discourse.ubuntu.com/t/contribute/26) for resources and tips on how to get started.

### Issues

Issues should be used to report problems with the software, request a new feature, or to discuss potential changes before a PR is created. When you create a new Issue, a template will be loaded that will guide you through collecting and providing the information we need to investigate.

If you find an Issue that addresses the problem you're having, please add your own reproduction information to the existing issue rather than creating a new one. Adding a [reaction](https://github.blog/2016-03-10-add-reactions-to-pull-requests-issues-and-comments/) can also help be indicating to our maintainers that a particular problem is affecting more than just the reporter.

### Pull Requests

PRs to our project are always welcome and can be a quick way to get your fix or improvement slated for the next release. In general, PRs should:

* Only fix/add the functionality in question **OR** address wide-spread whitespace/style issues, not both.
* Add unit or integration tests for fixed or changed functionality.
* Address a single concern in the least number of changed lines as possible.
* Include documentation in the repository.
* Be accompanied by a complete Pull Request template (loaded automatically when a PR is created).

For changes that address core functionality or would require breaking changes (e.g. a major release), it's best to open an Issue to discuss your proposal first. This is not required but can save time creating and reviewing changes.

In general, we follow the ["fork-and-pull" Git workflow](https://github.com/susam/gitpr)

1. Fork the repository to your own Github account
1. Clone the project to your machine
1. Create a branch locally with a succinct but descriptive name
1. Commit changes to the branch
1. Following any formatting and testing guidelines specific to this repo
1. Push changes to your fork
1. Open a PR in our repository and follow the PR template so that we can efficiently review the changes.

> PRs will trigger unit and integration tests with and without race detection, linting and formatting validations, static and security checks, freshness of generated files verification. All the tests must pass before merging in main branch.

## Contributing to the code

### Required dependencies and setup

The `ubuntu-proxy-manager` binary can be built using the following command from the root directory of the repository, which will place a binary with the same name in the current path:
```sh
go build ./cmd/ubuntu-proxy-manager
```

To test the program as a D-Bus activated systemd service, run the following commands to copy the D-Bus/polkit/systemd configuration files in their appropriate locations:

```sh
# D-Bus configuration
cp com.ubuntu.ProxyManager.conf /etc/dbus-1/system.d

# D-Bus service
cp com.ubuntu.ProxyManager.service /usr/share/dbus-1/system-services

# polkit configuration
cp com.ubuntu.ProxyManager.policy /usr/share/polkit-1/actions

# systemd service
cp ubuntu-proxy-manager.service /usr/lib/systemd/system
```

By default, the systemd service expects the `ubuntu-proxy-manager` binary to be located under `/usr/libexec`. This can be adjusted by modifying the the `ExecStart` directive of `ubuntu-proxy-manager.service` to something like `/path/to/repo/ubuntu-proxy-manager`.

Remember to run `systemctl daemon-reload` after any changes to the systemd configuration.

Once everything is into place, the D-Bus interface can be exercised via client APIs or helper programs such as `d-feet`, `gdbus` or directly via `dbus-send`.

### About the testsuite

The project includes a comprehensive testsuite made of unit and integration tests. All the tests must pass with and without the race detector.

You can run all tests with: `go test ./...` (add `-race` for race detection).

The test suite must pass before merging the PR to our main branch. Any new feature, change or fix must be covered by corresponding tests.

## Contributor Licence Agreement

It is required to sign the [Contributor Licence Agreement](https://ubuntu.com/legal/contributors) in order to contribute to this project.

An automated test is executed on PRs to check if it has been accepted.

This project is covered by the [GPL-3.0 License](LICENSE).

## Getting Help

Join us in the [Ubuntu Community](https://discourse.ubuntu.com/c/desktop/8) and post your question there with a descriptive tag.
